@@ -48,6 +48,9 @@ class AX25KISSInterface(Interface):
     serial   = None
 
     def __init__(self, owner, name, callsign, ssid, port, speed, databits, parity, stopbits, preamble, txtail, persistence, slottime, flow_control):
+        self.rxb = 0
+        self.txb = 0
+        
         self.serial   = None
         self.owner    = owner
         self.name     = name
@@ -62,13 +65,12 @@ class AX25KISSInterface(Interface):
         self.stopbits = stopbits
         self.timeout  = 100
         self.online   = False
-        # TODO: Sane default and make this configurable
-        # TODO: Changed to 25ms instead of 100ms, check it
-        self.txdelay  = 0.025
 
         self.packet_queue    = []
         self.flow_control    = flow_control
         self.interface_ready = False
+        self.flow_control_timeout = 5
+        self.flow_control_locked  = time.time()
 
         if (len(self.src_call) < 3 or len(self.src_call) > 6):
             raise ValueError("Invalid callsign for "+str(self))
@@ -189,14 +191,17 @@ class AX25KISSInterface(Interface):
 
     def processIncoming(self, data):
         if (len(data) > AX25.HEADER_SIZE):
+            self.rxb += len(data)
             self.owner.inbound(data[AX25.HEADER_SIZE:], self)
 
 
     def processOutgoing(self,data):
+        datalen = len(data)
         if self.online:
             if self.interface_ready:
                 if self.flow_control:
                     self.interface_ready = False
+                    self.flow_control_locked = time.time()
 
                 encoded_dst_ssid = bytes([0x60 | (self.dst_ssid << 1)])
                 encoded_src_ssid = bytes([0x60 | (self.src_ssid << 1) | 0x01])
@@ -223,11 +228,9 @@ class AX25KISSInterface(Interface):
                 data = data.replace(bytes([0xc0]), bytes([0xdb])+bytes([0xdc]))
                 kiss_frame = bytes([KISS.FEND])+bytes([0x00])+data+bytes([KISS.FEND])
 
-                if (self.txdelay > 0):
-                    RNS.log(str(self.name)+" delaying TX for "+str(self.txdelay)+" seconds", RNS.LOG_EXTREME)
-                    sleep(self.txdelay)
-
                 written = self.serial.write(kiss_frame)
+                self.txb += datalen
+
                 if written != len(kiss_frame):
                     if self.flow_control:
                         self.interface_ready = True
@@ -294,12 +297,21 @@ class AX25KISSInterface(Interface):
                         in_frame = False
                         command = KISS.CMD_UNKNOWN
                         escape = False
-                    sleep(0.08)
+                    sleep(0.05)
+
+                    if self.flow_control:
+                        if not self.interface_ready:
+                            if time.time() > self.flow_control_locked + self.flow_control_timeout:
+                                RNS.log("Interface "+str(self)+" is unlocking flow control due to time-out. This should not happen. Your hardware might have missed a flow-control READY command, or maybe it does not support flow-control.", RNS.LOG_WARNING)
+                                self.process_queue()
 
         except Exception as e:
             self.online = False
             RNS.log("A serial port error occurred, the contained exception was: "+str(e), RNS.LOG_ERROR)
-            RNS.log("The interface "+str(self.name)+" is now offline. Restart Reticulum to attempt reconnection.", RNS.LOG_ERROR)
+            RNS.log("The interface "+str(self)+" experienced an unrecoverable error and is being torn down. Restart Reticulum to attempt to open this interface again.", RNS.LOG_ERROR)
+
+            if RNS.Reticulum.panic_on_interface_error:
+                RNS.panic()
 
     def __str__(self):
         return "AX25KISSInterface["+self.name+"]"
\ No newline at end of file
