@@ -8,8 +8,12 @@ class Interface:
     name = None
 
     def __init__(self):
-        pass
+        self.rxb = 0
+        self.txb = 0
+        self.online = False
 
     def get_hash(self):
-        # TODO: Maybe expand this to something more unique
-        return RNS.Identity.fullHash(str(self).encode("utf-8"))
\ No newline at end of file
+        return RNS.Identity.full_hash(str(self).encode("utf-8"))
+
+    def detach(self):
+        pass
\ No newline at end of file
