@@ -5,6 +5,23 @@ import time
 import RNS
 
 class Packet:
+    """
+    The Packet class is used to create packet instances that can be sent
+    over a Reticulum network. Packets to will automatically be encrypted if
+    they are adressed to a ``RNS.Destination.SINGLE`` destination,
+    ``RNS.Destination.GROUP`` destination or a :ref:`RNS.Link<api-link>`.
+
+    For ``RNS.Destination.GROUP`` destinations, Reticulum will use the
+    pre-shared key configured for the destination.
+
+    For ``RNS.Destination.SINGLE`` destinations and :ref:`RNS.Link<api-link>`
+    destinations, reticulum will use ephemeral keys, and offers **Forward Secrecy**.
+
+    :param destination: A :ref:`RNS.Destination<api-destination>` instance to which the packet will be sent.
+    :param data: The data payload to be included in the packet as *bytes*.
+    :param create_receipt: Specifies whether a :ref:`RNS.PacketReceipt<api-packetreceipt>` should be created when instantiating the packet.
+    """
+
     # Packet types
     DATA         = 0x00     # Data packets
     ANNOUNCE     = 0x01     # Announces
@@ -19,7 +36,7 @@ class Packet:
     HEADER_4     = 0x03     # Reserved
     header_types = [HEADER_1, HEADER_2, HEADER_3, HEADER_4]
 
-    # Data packet context types
+    # Packet context types
     NONE           = 0x00   # Generic data packet
     RESOURCE       = 0x01   # Packet is part of a resource
     RESOURCE_ADV   = 0x02   # Packet is a resource advertisement
@@ -34,7 +51,8 @@ class Packet:
     PATH_RESPONSE  = 0x0B   # Packet is a response to a path request
     COMMAND        = 0x0C   # Packet is a command
     COMMAND_STATUS = 0x0D   # Packet is a status of an executed command
-    KEEPALIVE      = 0xFB   # Packet is a keepalive packet
+    KEEPALIVE      = 0xFA   # Packet is a keepalive packet
+    LINKIDENTIFY   = 0xFB   # Packet is a link peer identification proof
     LINKCLOSE      = 0xFC   # Packet is a link close message
     LINKPROOF      = 0xFD   # Packet is a link packet proof
     LRRTT          = 0xFE   # Packet is a link request round-trip time measurement
@@ -42,19 +60,24 @@ class Packet:
 
     # This is used to calculate allowable
     # payload sizes
-    HEADER_MAXSIZE = 23
+    HEADER_MAXSIZE = RNS.Reticulum.HEADER_MAXSIZE
     MDU            = RNS.Reticulum.MDU
 
-    # With an MTU of 500, the maximum RSA-encrypted
-    # amount of data we can send in a single packet
-    # is given by the below calculation; 258 bytes.
-    RSA_MDU   = math.floor(MDU/RNS.Identity.DECRYPT_CHUNKSIZE)*RNS.Identity.ENCRYPT_CHUNKSIZE
-    PLAIN_MDU = MDU
+    # With an MTU of 500, the maximum of data we can
+    # send in a single encrypted packet is given by
+    # the below calculation; 383 bytes.
+    ENCRYPTED_MDU  = math.floor((RNS.Reticulum.MDU-RNS.Identity.FERNET_OVERHEAD-RNS.Identity.KEYSIZE//16)/RNS.Identity.AES128_BLOCKSIZE)*RNS.Identity.AES128_BLOCKSIZE - 1
+    """
+    The maximum size of the payload data in a single encrypted packet 
+    """
+    PLAIN_MDU      = MDU
+    """
+    The maximum size of the payload data in a single unencrypted packet 
+    """
 
-    # TODO: This should be calculated
-    # more intelligently
-    # Default packet timeout
-    TIMEOUT      = 60
+    # This value is set at a reasonable
+    # level for a 1 Kb/s channel.
+    TIMEOUT_PER_HOP = 5
 
     def __init__(self, destination, data, packet_type = DATA, context = NONE, transport_type = RNS.Transport.BROADCAST, header_type = HEADER_1, transport_id = None, attached_interface = None, create_receipt = True):
         if destination != None:
@@ -70,7 +93,7 @@ class Packet:
             self.destination    = destination
             self.transport_id   = transport_id
             self.data           = data
-            self.flags          = self.getPackedFlags()
+            self.flags          = self.get_packed_flags()
 
             self.raw            = None
             self.packed         = False
@@ -91,7 +114,7 @@ class Packet:
         self.attached_interface = attached_interface
         self.receiving_interface = None
 
-    def getPackedFlags(self):
+    def get_packed_flags(self):
         if self.context == Packet.LRPROOF:
             packed_flags = (self.header_type << 6) | (self.transport_type << 4) | RNS.Destination.LINK | self.packet_type
         else:
@@ -114,6 +137,9 @@ class Packet:
                 if self.packet_type == Packet.ANNOUNCE:
                     # Announce packets are not encrypted
                     self.ciphertext = self.data
+                elif self.packet_type == Packet.LINKREQUEST:
+                    # Link request packets are not encrypted
+                    self.ciphertext = self.data
                 elif self.packet_type == Packet.PROOF and self.context == Packet.RESOURCE_PRF:
                     # Resource proofs are not encrypted
                     self.ciphertext = self.data
@@ -155,7 +181,8 @@ class Packet:
             raise IOError("Packet size of "+str(len(self.raw))+" exceeds MTU of "+str(self.MTU)+" bytes")
 
         self.packed = True
-        self.updateHash()
+        self.update_hash()
+
 
     def unpack(self):
         self.flags = self.raw[0]
@@ -178,12 +205,14 @@ class Packet:
             self.data = self.raw[13:]
 
         self.packed = False
-        self.updateHash()
+        self.update_hash()
 
-    # Sends the packet. Returns a receipt if one is generated,
-    # or None if no receipt is available. Returns False if the
-    # packet could not be sent.
     def send(self):
+        """
+        Sends the packet.
+        
+        :returns: A :ref:`RNS.PacketReceipt<api-packetreceipt>` instance if *create_receipt* was set to *True* when the packet was instantiated, if not returns *None*. If the packet could not be sent *False* is returned.
+        """
         if not self.sent:
             if self.destination.type == RNS.Destination.LINK:
                 if self.destination.status == RNS.Link.CLOSED:
@@ -208,6 +237,11 @@ class Packet:
             raise IOError("Packet was already sent")
 
     def resend(self):
+        """
+        Re-sends the packet.
+        
+        :returns: A :ref:`RNS.PacketReceipt<api-packetreceipt>` instance if *create_receipt* was set to *True* when the packet was instantiated, if not returns *None*. If the packet could not be sent *False* is returned.
+        """
         if self.sent:
             if RNS.Transport.outbound(self):
                 return self.receipt
@@ -230,25 +264,25 @@ class Packet:
 
     # Generates a special destination that allows Reticulum
     # to direct the proof back to the proved packet's sender
-    def generateProofDestination(self):
+    def generate_proof_destination(self):
         return ProofDestination(self)
 
-    def validateProofPacket(self, proof_packet):
-        return self.receipt.validateProofPacket(proof_packet)
+    def validate_proof_packet(self, proof_packet):
+        return self.receipt.validate_proof_packet(proof_packet)
 
-    def validateProof(self, proof):
-        return self.receipt.validateProof(proof)
+    def validate_proof(self, proof):
+        return self.receipt.validate_proof(proof)
 
-    def updateHash(self):
-        self.packet_hash = self.getHash()
+    def update_hash(self):
+        self.packet_hash = self.get_hash()
 
-    def getHash(self):
-        return RNS.Identity.fullHash(self.getHashablePart())
+    def get_hash(self):
+        return RNS.Identity.full_hash(self.get_hashable_part())
 
     def getTruncatedHash(self):
-        return RNS.Identity.truncatedHash(self.getHashablePart())
+        return RNS.Identity.truncated_hash(self.get_hashable_part())
 
-    def getHashablePart(self):
+    def get_hashable_part(self):
         hashable_part = bytes([self.raw[0] & 0b00001111])
         if self.header_type == Packet.HEADER_2:
             hashable_part += self.raw[12:]
@@ -259,7 +293,7 @@ class Packet:
 
 class ProofDestination:
     def __init__(self, packet):
-        self.hash = packet.getHash()[:10];
+        self.hash = packet.get_hash()[:10];
         self.type = RNS.Destination.SINGLE
 
     def encrypt(self, plaintext):
@@ -267,6 +301,12 @@ class ProofDestination:
 
 
 class PacketReceipt:
+    """
+    The PacketReceipt class is used to receive notifications about
+    :ref:`RNS.Packet<api-packet>` instances sent over the network. Instances
+    of this class are never created manually, but always returned from
+    the *send()* method of a :ref:`RNS.Packet<api-packet>` instance.
+    """
     # Receipt status constants
     FAILED    = 0x00
     SENT      = 0x01
@@ -279,22 +319,34 @@ class PacketReceipt:
 
     # Creates a new packet receipt from a sent packet
     def __init__(self, packet):
-        self.hash    = packet.getHash()
-        self.sent    = True
-        self.sent_at = time.time()
-        self.timeout = Packet.TIMEOUT
-        self.proved  = False
-        self.status  = PacketReceipt.SENT
-        self.destination = packet.destination
-        self.callbacks   = PacketReceiptCallbacks()
-        self.concluded_at = None
+        self.hash           = packet.get_hash()
+        self.truncated_hash = packet.getTruncatedHash()
+        self.sent           = True
+        self.sent_at        = time.time()
+        self.proved         = False
+        self.status         = PacketReceipt.SENT
+        self.destination    = packet.destination
+        self.callbacks      = PacketReceiptCallbacks()
+        self.concluded_at   = None
+
+        if packet.destination.type == RNS.Destination.LINK:
+            self.timeout    = packet.destination.rtt * packet.destination.traffic_timeout_factor
+        else:
+            self.timeout    = Packet.TIMEOUT_PER_HOP * RNS.Transport.hops_to(self.destination.hash)
+
+
+    def get_status(self):
+        """
+        :returns: The status of the associated :ref:`RNS.Packet<api-packet>` instance. Can be one of ``RNS.PacketReceipt.SENT``, ``RNS.PacketReceipt.DELIVERED``, ``RNS.PacketReceipt.FAILED`` or ``RNS.PacketReceipt.CULLED``. 
+        """
+        return self.status
 
     # Validate a proof packet
-    def validateProofPacket(self, proof_packet):
+    def validate_proof_packet(self, proof_packet):
         if hasattr(proof_packet, "link") and proof_packet.link:
             return self.validate_link_proof(proof_packet.data, proof_packet.link)
         else:
-            return self.validateProof(proof_packet.data)
+            return self.validate_proof(proof_packet.data)
 
     # Validate a raw proof for a link
     def validate_link_proof(self, proof, link):
@@ -336,7 +388,7 @@ class PacketReceipt:
             return False
 
     # Validate a raw proof
-    def validateProof(self, proof):
+    def validate_proof(self, proof):
         if len(proof) == PacketReceipt.EXPL_LENGTH:
             # This is an explicit proof
             proof_hash = proof[:RNS.Identity.HASHLENGTH//8]
@@ -373,14 +425,17 @@ class PacketReceipt:
         else:
             return False
 
-    def rtt(self):
+    def get_rtt(self):
+        """
+        :returns: The round-trip-time in seconds
+        """
         return self.concluded_at - self.sent_at
 
     def is_timed_out(self):
         return (self.sent_at+self.timeout < time.time())
 
     def check_timeout(self):
-        if self.is_timed_out():
+        if self.status == PacketReceipt.SENT and self.is_timed_out():
             if self.timeout == -1:
                 self.status = PacketReceipt.CULLED
             else:
@@ -392,21 +447,32 @@ class PacketReceipt:
                 thread = threading.Thread(target=self.callbacks.timeout, args=(self,))
                 thread.setDaemon(True)
                 thread.start()
-                #self.callbacks.timeout(self)
 
 
-    # Set the timeout in seconds
     def set_timeout(self, timeout):
+        """
+        Sets a timeout in seconds
+        
+        :param timeout: The timeout in seconds.
+        """
         self.timeout = float(timeout)
 
-    # Set a function that gets called when
-    # a successfull delivery has been proved
-    def delivery_callback(self, callback):
+    def set_delivery_callback(self, callback):
+        """
+        Sets a function that gets called if a successfull delivery has been proven.
+
+        :param callback: A *callable* with the signature *callback(packet_receipt)*
+        """
         self.callbacks.delivery = callback
 
     # Set a function that gets called if the
     # delivery times out
-    def timeout_callback(self, callback):
+    def set_timeout_callback(self, callback):
+        """
+        Sets a function that gets called if the delivery times out.
+
+        :param callback: A *callable* with the signature *callback(packet_receipt)*
+        """
         self.callbacks.timeout = callback
 
 class PacketReceiptCallbacks:
