@@ -8,6 +8,18 @@ from .vendor import umsgpack as umsgpack
 from time import sleep
 
 class Resource:
+    """
+    The Resource class allows transferring arbitrary amounts
+    of data over a link. It will automatically handle sequencing,
+    compression, coordination and checksumming.
+
+    :param data: The data to be transferred. Can be *bytes* or an open *file handle*. See the :ref:`Filetransfer Example<example-filetransfer>` for details.
+    :param link: The :ref:`RNS.Link<api-link>` instance on which to transfer the data.
+    :param advertise: Optional. Whether to automatically advertise the resource. Can be *True* or *False*.
+    :param auto_compress: Optional. Whether to auto-compress the resource. Can be *True* or *False*.
+    :param callback: An optional *callable* with the signature *callback(resource)*. Will be called when the resource transfer concludes.
+    :param progress_callback: An optional *callable* with the signature *callback(resource)*. Will be called whenever the resource transfer progress is updated.
+    """
     WINDOW_FLEXIBILITY   = 4
     WINDOW_MIN           = 1
     WINDOW_MAX           = 10
@@ -25,39 +37,47 @@ class Resource:
     # defined as a Raspberry Pi, which should
     # be able to compress, encrypt and hash-map
     # the resource in about 10 seconds.
-    MAX_EFFICIENT_SIZE   = 16 * 1024 * 1024
-
+    #
+    # This constant will be used when determining
+    # how to sequence the sending of large resources.
+    #
+    # Capped at 16777215 (0xFFFFFF) per segment to
+    # fit in 3 bytes in resource advertisements.
+    MAX_EFFICIENT_SIZE      = 16 * 1024 * 1024 - 1
+    RESPONSE_MAX_GRACE_TIME = 10
+    
     # The maximum size to auto-compress with
     # bz2 before sending.
     AUTO_COMPRESS_MAX_SIZE = MAX_EFFICIENT_SIZE
 
-    # TODO: Should be allocated more
-    # intelligently
-    # TODO: Set higher
-    MAX_RETRIES       = 5
-    SENDER_GRACE_TIME = 10
-    RETRY_GRACE_TIME  = 0.25
+    PART_TIMEOUT_FACTOR           = 4
+    PART_TIMEOUT_FACTOR_AFTER_RTT = 2
+    MAX_RETRIES                   = 5
+    SENDER_GRACE_TIME             = 10
+    RETRY_GRACE_TIME              = 0.25
+
+    WATCHDOG_MAX_SLEEP            = 1
 
     HASHMAP_IS_NOT_EXHAUSTED = 0x00
     HASHMAP_IS_EXHAUSTED = 0xFF
 
     # Status constants
-    NONE             = 0x00
-    QUEUED             = 0x01
-    ADVERTISED         = 0x02
+    NONE            = 0x00
+    QUEUED          = 0x01
+    ADVERTISED      = 0x02
     TRANSFERRING    = 0x03
     AWAITING_PROOF  = 0x04
     ASSEMBLING      = 0x05
     COMPLETE        = 0x06
-    FAILED            = 0x07
-    CORRUPT            = 0x08
+    FAILED          = 0x07
+    CORRUPT         = 0x08
 
     @staticmethod
-    def accept(advertisement_packet, callback=None, progress_callback = None):
+    def accept(advertisement_packet, callback=None, progress_callback = None, request_id = None):
         try:
             adv = ResourceAdvertisement.unpack(advertisement_packet.plaintext)
 
-            resource = Resource(None, advertisement_packet.link)
+            resource = Resource(None, advertisement_packet.link, request_id = request_id)
             resource.status = Resource.TRANSFERRING
 
             resource.flags               = adv.f
@@ -102,7 +122,8 @@ class Resource:
             resource.link.register_incoming_resource(resource)
 
             RNS.log("Accepting resource advertisement for "+RNS.prettyhexrep(resource.hash), RNS.LOG_DEBUG)
-            resource.link.callbacks.resource_started(resource)
+            if resource.link.callbacks.resource_started != None:
+                resource.link.callbacks.resource_started(resource)
 
             resource.hashmap_update(0, resource.hashmap_raw)
 
@@ -111,14 +132,12 @@ class Resource:
             return resource
         except Exception as e:
             RNS.log("Could not decode resource advertisement, dropping resource", RNS.LOG_DEBUG)
-            # TODO: Remove
-            raise e
             return None
 
     # Create a resource for transmission to a remote destination
     # The data passed can be either a bytes-array or a file opened
     # in binary read mode.
-    def __init__(self, data, link, advertise=True, auto_compress=True, must_compress=False, callback=None, progress_callback=None, segment_index = 1, original_hash = None):
+    def __init__(self, data, link, advertise=True, auto_compress=True, callback=None, progress_callback=None, timeout = None, segment_index = 1, original_hash = None, request_id = None, is_response = False):
         data_size = None
         resource_data = None
         if hasattr(data, "read"):
@@ -165,24 +184,32 @@ class Resource:
         self.link = link
         self.max_retries = Resource.MAX_RETRIES
         self.retries_left = self.max_retries
-        self.default_timeout = self.link.default_timeout
-        self.timeout_factor = self.link.timeout_factor
+        self.timeout_factor = self.link.traffic_timeout_factor
+        self.part_timeout_factor = Resource.PART_TIMEOUT_FACTOR
         self.sender_grace_time = Resource.SENDER_GRACE_TIME
         self.hmu_retry_ok = False
         self.watchdog_lock = False
         self.__watchdog_job_id = 0
         self.__progress_callback = progress_callback
         self.rtt = None
+        self.request_id = request_id
+        self.is_response = is_response
 
+        self.req_hashlist = []
         self.receiver_min_consecutive_height = 0
 
+        if timeout != None:
+            self.timeout = timeout
+        else:
+            self.timeout = self.link.rtt * self.link.traffic_timeout_factor
+
         if data != None:
             self.initiator         = True
             self.callback          = callback
             self.uncompressed_data = data
 
             compression_began = time.time()
-            if must_compress or (auto_compress and len(self.uncompressed_data) < Resource.AUTO_COMPRESS_MAX_SIZE):
+            if (auto_compress and len(self.uncompressed_data) < Resource.AUTO_COMPRESS_MAX_SIZE):
                 RNS.log("Compressing resource data...", RNS.LOG_DEBUG)
                 self.compressed_data   = bz2.compress(self.uncompressed_data)
                 RNS.log("Compression completed in "+str(round(time.time()-compression_began, 3))+" seconds", RNS.LOG_DEBUG)
@@ -197,7 +224,7 @@ class Resource:
                 RNS.log("Compression saved "+str(saved_bytes)+" bytes, sending compressed", RNS.LOG_DEBUG)
 
                 self.data  = b""
-                self.data += RNS.Identity.getRandomHash()[:Resource.RANDOM_HASH_SIZE]
+                self.data += RNS.Identity.get_random_hash()[:Resource.RANDOM_HASH_SIZE]
                 self.data += self.compressed_data
                 
                 self.compressed = True
@@ -205,7 +232,7 @@ class Resource:
 
             else:
                 self.data  = b""
-                self.data += RNS.Identity.getRandomHash()[:Resource.RANDOM_HASH_SIZE]
+                self.data += RNS.Identity.get_random_hash()[:Resource.RANDOM_HASH_SIZE]
                 self.data += self.uncompressed_data
                 self.uncompressed_data = self.data
 
@@ -218,11 +245,8 @@ class Resource:
             # make optimal use of packet MTU on an entire
             # encrypted stream. The Resource instance will
             # use it's underlying link directly to encrypt.
-            if not self.link.encryption_disabled():
-                self.data = self.link.encrypt(self.data)
-                self.encrypted = True
-            else:
-                self.encrypted = False
+            self.data = self.link.encrypt(self.data)
+            self.encrypted = True
 
             self.size = len(self.data)
             self.sent_parts = 0
@@ -233,9 +257,10 @@ class Resource:
                 hashmap_computation_began = time.time()
                 RNS.log("Starting resource hashmap computation with "+str(hashmap_entries)+" entries...", RNS.LOG_DEBUG)
 
-                self.random_hash       = RNS.Identity.getRandomHash()[:Resource.RANDOM_HASH_SIZE]
-                self.hash = RNS.Identity.fullHash(data+self.random_hash)
-                self.expected_proof = RNS.Identity.fullHash(data+self.hash)
+                self.random_hash       = RNS.Identity.get_random_hash()[:Resource.RANDOM_HASH_SIZE]
+                self.hash = RNS.Identity.full_hash(data+self.random_hash)
+                self.truncated_hash = RNS.Identity.truncated_hash(data+self.random_hash)
+                self.expected_proof = RNS.Identity.full_hash(data+self.hash)
 
                 if original_hash == None:
                     self.original_hash = self.hash
@@ -247,7 +272,7 @@ class Resource:
                 collision_guard_list = []
                 for i in range(0,hashmap_entries):
                     data = self.data[i*Resource.SDU:(i+1)*Resource.SDU]
-                    map_hash = self.getMapHash(data)
+                    map_hash = self.get_map_hash(data)
 
                     if map_hash in collision_guard_list:
                         RNS.log("Found hash collision in resource map, remapping...", RNS.LOG_VERBOSE)
@@ -295,14 +320,18 @@ class Resource:
             self.waiting_for_hmu = False
             self.request_next()
 
-    def getMapHash(self, data):
+    def get_map_hash(self, data):
         # TODO: This will break if running unencrypted,
         # uncompressed transfers on streams with long blocks
         # of identical bytes. Doing so would be very silly
         # anyways but maybe it should be handled gracefully.
-        return RNS.Identity.fullHash(data+self.random_hash)[:Resource.MAPHASH_LEN]
+        return RNS.Identity.full_hash(data+self.random_hash)[:Resource.MAPHASH_LEN]
 
     def advertise(self):
+        """
+        Advertise the resource. If the other end of the link accepts
+        the resource advertisement it will begin transferring.
+        """
         thread = threading.Thread(target=self.__advertise_job)
         thread.setDaemon(True)
         thread.start()
@@ -345,7 +374,7 @@ class Resource:
             sleep_time = None
 
             if self.status == Resource.ADVERTISED:
-                sleep_time = (self.adv_sent+self.default_timeout)-time.time()
+                sleep_time = (self.adv_sent+self.timeout)-time.time()
                 if sleep_time < 0:
                     if self.retries_left <= 0:
                         RNS.log("Resource transfer timeout after sending advertisement", RNS.LOG_DEBUG)
@@ -366,8 +395,24 @@ class Resource:
 
             elif self.status == Resource.TRANSFERRING:
                 if not self.initiator:
-                    rtt = self.link.rtt if self.rtt == None else self.rtt
-                    sleep_time = self.last_activity + (rtt*self.timeout_factor) + Resource.RETRY_GRACE_TIME - time.time()
+
+                    if self.rtt == None:
+                        rtt = self.link.rtt
+                    else:
+                        rtt = self.rtt
+
+                    window_remaining = self.outstanding_parts
+
+                    sleep_time = self.last_activity + (rtt*(self.part_timeout_factor+window_remaining)) + Resource.RETRY_GRACE_TIME - time.time()
+                    
+                    # TODO: Remove debug info
+                    # RNS.log("rtt   "+str(rtt))
+                    # RNS.log("ptof  "+str(self.part_timeout_factor))
+                    # RNS.log("wait  "+str((rtt*self.part_timeout_factor) + Resource.RETRY_GRACE_TIME))
+                    # RNS.log("sleep "+str(sleep_time))
+                    # RNS.log("wndw  "+str(self.window))
+                    # RNS.log("wndwr "+str(window_remaining))
+                    # RNS.log("")
 
                     if sleep_time < 0:
                         if self.retries_left > 0:
@@ -418,7 +463,7 @@ class Resource:
                 self.cancel()
             
             if sleep_time != None:
-                sleep(sleep_time)
+                sleep(min(sleep_time, Resource.WATCHDOG_MAX_SLEEP))
 
     def assemble(self):
         if not self.status == Resource.FAILED:
@@ -439,7 +484,7 @@ class Resource:
                 else:
                     self.data = data
 
-                calculated_hash = RNS.Identity.fullHash(self.data+self.random_hash)
+                calculated_hash = RNS.Identity.full_hash(self.data+self.random_hash)
 
                 if calculated_hash == self.hash:
                     self.file = open(self.storagepath, "ab")
@@ -476,7 +521,7 @@ class Resource:
     def prove(self):
         if not self.status == Resource.FAILED:
             try:
-                proof = RNS.Identity.fullHash(self.data+self.hash)
+                proof = RNS.Identity.full_hash(self.data+self.hash)
                 proof_data = self.hash+proof
                 proof_packet = RNS.Packet(self.link, proof_data, packet_type=RNS.Packet.PROOF, context=RNS.Packet.RESOURCE_PRF)
                 proof_packet.send()
@@ -485,7 +530,7 @@ class Resource:
                 RNS.log("The contained exception was: "+str(e), RNS.LOG_DEBUG)
                 self.cancel()
 
-    def validateProof(self, proof_data):
+    def validate_proof(self, proof_data):
         if not self.status == Resource.FAILED:
             if len(proof_data) == RNS.Identity.HASHLENGTH//8*2:
                 if proof_data[RNS.Identity.HASHLENGTH//8:] == self.expected_proof:
@@ -517,16 +562,20 @@ class Resource:
         if self.req_resp == None:
             self.req_resp = self.last_activity
             rtt = self.req_resp-self.req_sent
+
+            self.part_timeout_factor = Resource.PART_TIMEOUT_FACTOR_AFTER_RTT
             if self.rtt == None:
-                self.rtt = rtt
+                self.rtt = self.link.rtt
                 self.watchdog_job()
-            elif self.rtt < rtt:
-                self.rtt = rtt
+            elif rtt < self.rtt:
+                self.rtt = max(self.rtt - self.rtt*0.05, rtt)
+            elif rtt > self.rtt:
+                self.rtt = min(self.rtt + self.rtt*0.05, rtt)
 
         if not self.status == Resource.FAILED:
             self.status = Resource.TRANSFERRING
             part_data = packet.data
-            part_hash = self.getMapHash(part_data)
+            part_hash = self.get_map_hash(part_data)
 
             i = self.consecutive_completed_height
             for map_hash in self.hashmap[self.consecutive_completed_height:self.consecutive_completed_height+self.window]:
@@ -546,14 +595,21 @@ class Resource:
                             self.consecutive_completed_height = cp
                             cp += 1
 
+                        if self.__progress_callback != None:
+                            self.__progress_callback(self)
+
+                        # TODO: Remove debug info
+                        # RNS.log("outstanding_parts "+str(self.outstanding_parts))
+                        # RNS.log("total_parts "+str(self.total_parts))
+                        # RNS.log("received_count "+str(self.received_count))
+
                 i += 1
 
             self.receiving_part = False
 
-            if self.__progress_callback != None:
-                self.__progress_callback(self)
-
-            if self.outstanding_parts == 0 and self.received_count == self.total_parts:
+            # TODO: Remove
+            #if self.outstanding_parts == 0 and self.received_count == self.total_parts:
+            if self.received_count == self.total_parts:
                 self.assemble()
             elif self.outstanding_parts == 0:
                 # TODO: Figure out if there is a mathematically
@@ -671,6 +727,7 @@ class Resource:
                 if part_index % ResourceAdvertisement.HASHMAP_MAX_LEN != 0:
                     RNS.log("Resource sequencing error, cancelling transfer!", RNS.LOG_ERROR)
                     self.cancel()
+                    return
                 else:
                     segment = part_index // ResourceAdvertisement.HASHMAP_MAX_LEN
 
@@ -700,6 +757,9 @@ class Resource:
                 self.__progress_callback(self)
 
     def cancel(self):
+        """
+        Cancels transferring the resource.
+        """
         if self.status < Resource.COMPLETE:
             self.status = Resource.FAILED
             if self.initiator:
@@ -717,13 +777,17 @@ class Resource:
                 self.link.resource_concluded(self)
                 self.callback(self)
 
+    def set_callback(self, callback):
+        self.callback = callback
+
     def progress_callback(self, callback):
         self.__progress_callback = callback
 
-    def progress(self):
+    def get_progress(self):
+        """
+        :returns: The current progress of the resource transfer as a *float* between 0.0 and 1.0.
+        """
         if self.initiator:
-            # TODO: Remove
-            # progress = self.sent_parts / len(self.parts)
             self.processed_parts  = (self.segment_index-1)*math.ceil(Resource.MAX_EFFICIENT_SIZE/Resource.SDU)
             self.processed_parts += self.sent_parts
             self.progress_total_parts = float(self.grand_total_parts)
@@ -740,28 +804,82 @@ class Resource:
         return progress
 
     def __str__(self):
-        return RNS.prettyhexrep(self.hash)+str(self.link)
+        return "<"+RNS.hexrep(self.hash)+"/"+RNS.hexrep(self.link.link_id)+">"
 
 
 class ResourceAdvertisement:
-    HASHMAP_MAX_LEN      = 75
+    OVERHEAD             = 128
+    HASHMAP_MAX_LEN      = math.floor((RNS.Link.MDU-OVERHEAD)/Resource.MAPHASH_LEN)
     COLLISION_GUARD_SIZE = 2*Resource.WINDOW_MAX+HASHMAP_MAX_LEN
 
-    def __init__(self, resource=None):
+    assert HASHMAP_MAX_LEN > 0, "The configured MTU is too small to include any map hashes in resource advertisments"
+
+    @staticmethod
+    def is_request(advertisement_packet):
+        adv = ResourceAdvertisement.unpack(advertisement_packet.plaintext)
+        if adv.q != None and adv.u:
+            return True
+        else:
+            return False
+
+
+    @staticmethod
+    def is_response(advertisement_packet):
+        adv = ResourceAdvertisement.unpack(advertisement_packet.plaintext)
+
+        if adv.q != None and adv.p:
+            return True
+        else:
+            return False
+
+
+    @staticmethod
+    def get_request_id(advertisement_packet):
+        adv = ResourceAdvertisement.unpack(advertisement_packet.plaintext)
+        return adv.q
+
+
+    @staticmethod
+    def get_transfer_size(advertisement_packet):
+        adv = ResourceAdvertisement.unpack(advertisement_packet.plaintext)
+        return adv.t
+
+
+    @staticmethod
+    def get_size(advertisement_packet):
+        adv = ResourceAdvertisement.unpack(advertisement_packet.plaintext)
+        return adv.d
+
+
+    def __init__(self, resource=None, request_id=None, is_response=False):
         if resource != None:
-            self.t = resource.size                             # Transfer size
-            self.d = resource.total_size                       # Total uncompressed data size
-            self.n = len(resource.parts)                       # Number of parts
-            self.h = resource.hash                             # Resource hash
-            self.r = resource.random_hash                      # Resource random hash
-            self.o = resource.original_hash                    # First-segment hash
-            self.m = resource.hashmap                          # Resource hashmap
-            self.c = resource.compressed                       # Compression flag
-            self.e = resource.encrypted                        # Encryption flag
-            self.s = resource.split                            # Split flag
-            self.i = resource.segment_index                    # Segment index
-            self.l = resource.total_segments                   # Total segments
-            self.f = 0x00 | self.s << 2 | self.c << 1 | self.e # Flags
+            self.t = resource.size              # Transfer size
+            self.d = resource.total_size        # Total uncompressed data size
+            self.n = len(resource.parts)        # Number of parts
+            self.h = resource.hash              # Resource hash
+            self.r = resource.random_hash       # Resource random hash
+            self.o = resource.original_hash     # First-segment hash
+            self.m = resource.hashmap           # Resource hashmap
+            self.c = resource.compressed        # Compression flag
+            self.e = resource.encrypted         # Encryption flag
+            self.s = resource.split             # Split flag
+            self.i = resource.segment_index     # Segment index
+            self.l = resource.total_segments    # Total segments
+            self.q = resource.request_id        # ID of associated request
+            self.u = False                      # Is request flag
+            self.p = False                      # Is response flag
+
+            if self.q != None:
+                if not resource.is_response:
+                    self.u = True
+                    self.p = False
+                else:
+                    self.u = False
+                    self.p = True
+
+            # Flags
+            self.f = 0x00 | self.p << 4 | self.u << 3 | self.s << 2 | self.c << 1 | self.e
+
 
     def pack(self, segment=0):
         hashmap_start = segment*ResourceAdvertisement.HASHMAP_MAX_LEN
@@ -780,12 +898,14 @@ class ResourceAdvertisement:
             "o": self.o,    # Original hash
             "i": self.i,    # Segment index
             "l": self.l,    # Total segments
+            "q": self.q,    # Request ID
             "f": self.f,    # Resource flags
             "m": hashmap
         }
 
         return umsgpack.packb(dictionary)
 
+
     @staticmethod
     def unpack(data):
         dictionary = umsgpack.unpackb(data)
@@ -801,8 +921,11 @@ class ResourceAdvertisement:
         adv.f = dictionary["f"]
         adv.i = dictionary["i"]
         adv.l = dictionary["l"]
+        adv.q = dictionary["q"]
         adv.e = True if (adv.f & 0x01) == 0x01 else False
         adv.c = True if ((adv.f >> 1) & 0x01) == 0x01 else False
         adv.s = True if ((adv.f >> 2) & 0x01) == 0x01 else False
+        adv.u = True if ((adv.f >> 3) & 0x01) == 0x01 else False
+        adv.p = True if ((adv.f >> 4) & 0x01) == 0x01 else False
 
         return adv
\ No newline at end of file
