@@ -1,7 +1,10 @@
 from .Interfaces import *
-import configparser
 from .vendor.configobj import ConfigObj
+import configparser
+import multiprocessing.connection
 import RNS
+import signal
+import threading
 import atexit
 import struct
 import array
@@ -10,13 +13,59 @@ import os
 import RNS
 
 class Reticulum:
+    """
+    This class is used to initialise access to Reticulum within a
+    program. You must create exactly one instance of this class before
+    carrying out any other RNS operations, such as creating destinations
+    or sending traffic. Every independently executed program must create
+    their own instance of the Reticulum class, but Reticulum will
+    automatically handle inter-program communication on the same system,
+    and expose all connected programs to external interfaces as well.
+
+    As soon as an instance of this class is created, Reticulum will start
+    opening and configuring any hardware devices specified in the supplied
+    configuration.
+
+    Currently the first running instance must be kept running while other
+    local instances are connected, as the first created instance will
+    act as a master instance that directly communicates with external
+    hardware such as modems, TNCs and radios. If a master instance is
+    asked to exit, it will not exit until all client processes have
+    terminated (unless killed forcibly).
+
+    If you are running Reticulum on a system with several different
+    programs that use RNS starting and terminating at different times,
+    it will be advantageous to run a master RNS instance as a daemon for
+    other programs to use on demand.
+    """
+
+    # Future minimum will probably be locked in at 244 bytes to support
+    # networks with segments of different MTUs. Absolute minimum is 211.
     MTU            = 500
-    HEADER_MAXSIZE = 23
+    """
+    The MTU that Reticulum adheres to, and will expect other peers to
+    adhere to. By default, the MTU is 500 bytes. In custom RNS network
+    implementations, it is possible to change this value, but doing so will
+    completely break compatibility with all other RNS networks. An identical
+    MTU is a prerequisite for peers to communicate in the same network.
+
+    Unless you really know what you are doing, the MTU should be left at
+    the default value.
+    """
+
+    # Length of truncated hashes in bits.
+    TRUNCATED_HASHLENGTH = 80
+
+    HEADER_MINSIZE = 2+1+(TRUNCATED_HASHLENGTH//8)*1
+    HEADER_MAXSIZE = 2+1+(TRUNCATED_HASHLENGTH//8)*2
+    
     MDU            = MTU - HEADER_MAXSIZE
 
     router         = None
     config         = None
     
+    # The default configuration path will be expanded to a directory
+    # named ".reticulum" inside the current users home directory
     configdir    = os.path.expanduser("~")+"/.reticulum"
     configpath   = ""
     storagepath  = ""
@@ -24,10 +73,29 @@ class Reticulum:
     
     @staticmethod
     def exit_handler():
-        RNS.Transport.exitHandler()
-        RNS.Identity.exitHandler()
+        # This exit handler is called whenever Reticulum is asked to
+        # shut down, and will in turn call exit handlers in other
+        # classes, saving necessary information to disk and carrying
+        # out cleanup operations.
+
+        RNS.Transport.exit_handler()
+        RNS.Identity.exit_handler()
+
+    @staticmethod
+    def sigint_handler(signal, frame):
+        RNS.Transport.detach_interfaces()
+        RNS.exit()
+
+
+    def __init__(self,configdir=None, loglevel=None):
+        """
+        Initialises and starts a Reticulum instance. This must be
+        done before any other operations, and Reticulum will not
+        pass any traffic before being instantiated.
+
+        :param configdir: Full path to a Reticulum configuration directory.
+        """
 
-    def __init__(self,configdir=None):
         if configdir != None:
             Reticulum.configdir = configdir
         
@@ -36,12 +104,24 @@ class Reticulum:
         Reticulum.cachepath     = Reticulum.configdir+"/storage/cache"
         Reticulum.resourcepath  = Reticulum.configdir+"/storage/resources"
 
-        Reticulum.__allow_unencrypted = False
         Reticulum.__transport_enabled = False
         Reticulum.__use_implicit_proof = True
 
+        Reticulum.panic_on_interface_error = False
+
         self.local_interface_port = 37428
-        self.share_instance = True
+        self.local_control_port   = 37429
+        self.share_instance       = True
+        self.rpc_listener         = None
+
+        self.requested_loglevel = loglevel
+        if self.requested_loglevel != None:
+            if self.requested_loglevel > RNS.LOG_EXTREME:
+                self.requested_loglevel = RNS.LOG_EXTREME
+            if self.requested_loglevel < RNS.LOG_CRITICAL:
+                self.requested_loglevel = RNS.LOG_CRITICAL
+
+            RNS.loglevel = self.requested_loglevel
 
         self.is_shared_instance = False
         self.is_connected_to_shared_instance = False
@@ -59,26 +139,37 @@ class Reticulum:
         if os.path.isfile(self.configpath):
             try:
                 self.config = ConfigObj(self.configpath)
-                RNS.log("Configuration loaded from "+self.configpath)
             except Exception as e:
                 RNS.log("Could not parse the configuration at "+self.configpath, RNS.LOG_ERROR)
                 RNS.log("Check your configuration file for errors!", RNS.LOG_ERROR)
                 RNS.panic()
         else:
             RNS.log("Could not load config file, creating default configuration file...")
-            self.createDefaultConfig()
+            self.__create_default_config()
             RNS.log("Default config file created. Make any necessary changes in "+Reticulum.configdir+"/config and start Reticulum again.")
             RNS.log("Exiting now!")
             exit(1)
 
-        self.applyConfig()
-        RNS.Identity.loadKnownDestinations()
+        self.__apply_config()
+        RNS.log("Configuration loaded from "+self.configpath, RNS.LOG_VERBOSE)
+        
+        RNS.Identity.load_known_destinations()
 
         RNS.Transport.start(self)
 
-        atexit.register(Reticulum.exit_handler)
+        self.rpc_addr = ("127.0.0.1", self.local_control_port)
+        self.rpc_key  = RNS.Identity.full_hash(RNS.Transport.identity.get_private_key())
+        
+        if self.is_shared_instance:
+            self.rpc_listener = multiprocessing.connection.Listener(self.rpc_addr, authkey=self.rpc_key)
+            thread = threading.Thread(target=self.rpc_loop)
+            thread.setDaemon(True)
+            thread.start()
 
-    def start_local_interface(self):
+        atexit.register(Reticulum.exit_handler)
+        signal.signal(signal.SIGINT, Reticulum.sigint_handler)
+
+    def __start_local_interface(self):
         if self.share_instance:
             try:
                 interface = LocalInterface.LocalServerInterface(
@@ -87,6 +178,7 @@ class Reticulum:
                 )
                 interface.OUT = True
                 RNS.Transport.interfaces.append(interface)
+                
                 self.is_shared_instance = True
                 RNS.log("Started shared instance interface: "+str(interface), RNS.LOG_DEBUG)
             except Exception as e:
@@ -101,6 +193,7 @@ class Reticulum:
                     self.is_shared_instance = False
                     self.is_standalone_instance = False
                     self.is_connected_to_shared_instance = True
+                    Reticulum.__transport_enabled = False
                     RNS.log("Connected to local shared instance via: "+str(interface), RNS.LOG_DEBUG)
                 except Exception as e:
                     RNS.log("Local shared instance appears to be running, but it could not be connected", RNS.LOG_ERROR)
@@ -113,11 +206,11 @@ class Reticulum:
             self.is_standalone_instance = True
             self.is_connected_to_shared_instance = False
 
-    def applyConfig(self):
+    def __apply_config(self):
         if "logging" in self.config:
             for option in self.config["logging"]:
                 value = self.config["logging"][option]
-                if option == "loglevel":
+                if option == "loglevel" and self.requested_loglevel == None:
                     RNS.loglevel = int(value)
                     if RNS.loglevel < 0:
                         RNS.loglevel = 0
@@ -133,32 +226,25 @@ class Reticulum:
                 if option == "shared_instance_port":
                     value = int(self.config["reticulum"][option])
                     self.local_interface_port = value
+                if option == "instance_control_port":
+                    value = int(self.config["reticulum"][option])
+                    self.local_control_port = value
                 if option == "enable_transport":
                     v = self.config["reticulum"].as_bool(option)
                     if v == True:
                         Reticulum.__transport_enabled = True
+                if option == "panic_on_interface_error":
+                    v = self.config["reticulum"].as_bool(option)
+                    if v == True:
+                        Reticulum.panic_on_interface_error = True
                 if option == "use_implicit_proof":
                     v = self.config["reticulum"].as_bool(option)
                     if v == True:
                         Reticulum.__use_implicit_proof = True
                     if v == False:
                         Reticulum.__use_implicit_proof = False
-                if option == "allow_unencrypted":
-                    v = self.config["reticulum"].as_bool(option)
-                    if v == True:
-                        RNS.log("", RNS.LOG_CRITICAL)
-                        RNS.log("! ! !     ! ! !     ! ! !", RNS.LOG_CRITICAL)
-                        RNS.log("", RNS.LOG_CRITICAL)
-                        RNS.log("Danger! Encryptionless links have been allowed in the config file!", RNS.LOG_CRITICAL)
-                        RNS.log("Beware of the consequences! Any data sent over a link can potentially be intercepted,", RNS.LOG_CRITICAL)
-                        RNS.log("read and modified! If you are not absolutely sure that you want this,", RNS.LOG_CRITICAL)
-                        RNS.log("you should exit Reticulum NOW and change your config file!", RNS.LOG_CRITICAL)
-                        RNS.log("", RNS.LOG_CRITICAL)
-                        RNS.log("! ! !     ! ! !     ! ! !", RNS.LOG_CRITICAL)
-                        RNS.log("", RNS.LOG_CRITICAL)
-                        Reticulum.__allow_unencrypted = True
 
-        self.start_local_interface()
+        self.__start_local_interface()
 
         if self.is_shared_instance or self.is_standalone_instance:
             interface_names = []
@@ -169,13 +255,27 @@ class Reticulum:
                     try:
                         if ("interface_enabled" in c) and c.as_bool("interface_enabled") == True:
                             if c["type"] == "UDPInterface":
+                                device       = c["device"] if "device" in c else None
+                                port         = int(c["port"]) if "port" in c else None
+                                listen_ip    = c["listen_ip"] if "listen_ip" in c else None
+                                listen_port  = int(c["listen_port"]) if "listen_port" in c else None
+                                forward_ip   = c["forward_ip"] if "forward_ip" in c else None
+                                forward_port = int(c["forward_port"]) if "forward_port" in c else None
+
+                                if port != None:
+                                    if listen_port == None:
+                                        listen_port = port
+                                    if forward_port == None:
+                                        forward_port = port
+
                                 interface = UDPInterface.UDPInterface(
                                     RNS.Transport,
                                     name,
-                                    c["listen_ip"],
-                                    int(c["listen_port"]),
-                                    c["forward_ip"],
-                                    int(c["forward_port"])
+                                    device,
+                                    listen_ip,
+                                    listen_port,
+                                    forward_ip,
+                                    forward_port
                                 )
 
                                 if "outgoing" in c and c.as_bool("outgoing") == True:
@@ -187,11 +287,20 @@ class Reticulum:
 
 
                             if c["type"] == "TCPServerInterface":
+                                device       = c["device"] if "device" in c else None
+                                port         = int(c["port"]) if "port" in c else None
+                                listen_ip    = c["listen_ip"] if "listen_ip" in c else None
+                                listen_port  = int(c["listen_port"]) if "listen_port" in c else None
+
+                                if port != None:
+                                    listen_port = port
+
                                 interface = TCPInterface.TCPServerInterface(
                                     RNS.Transport,
                                     name,
-                                    c["listen_ip"],
-                                    int(c["listen_port"])
+                                    device,
+                                    listen_ip,
+                                    listen_port
                                 )
 
                                 if "outgoing" in c and c.as_bool("outgoing") == True:
@@ -238,7 +347,7 @@ class Reticulum:
                                     stopbits
                                 )
 
-                                if "outgoing" in c and c["outgoing"].lower() == "true":
+                                if "outgoing" in c and c.as_bool("outgoing") == True:
                                     interface.OUT = True
                                 else:
                                     interface.OUT = False
@@ -279,7 +388,7 @@ class Reticulum:
                                     beacon_data
                                 )
 
-                                if "outgoing" in c and c["outgoing"].lower() == "true":
+                                if "outgoing" in c and c.as_bool("outgoing") == True:
                                     interface.OUT = True
                                 else:
                                     interface.OUT = False
@@ -321,7 +430,7 @@ class Reticulum:
                                     flow_control
                                 )
 
-                                if "outgoing" in c and c["outgoing"].lower() == "true":
+                                if "outgoing" in c and c.as_bool("outgoing") == True:
                                     interface.OUT = True
                                 else:
                                     interface.OUT = False
@@ -357,14 +466,14 @@ class Reticulum:
                                     id_callsign = id_callsign
                                 )
 
-                                if "outgoing" in c and c["outgoing"].lower() == "true":
+                                if "outgoing" in c and c.as_bool("outgoing") == True:
                                     interface.OUT = True
                                 else:
                                     interface.OUT = False
 
                                 RNS.Transport.interfaces.append(interface)
                         else:
-                            RNS.log("Skipping disabled interface \""+name+"\"", RNS.LOG_NOTICE)
+                            RNS.log("Skipping disabled interface \""+name+"\"", RNS.LOG_INFO)
 
                     except Exception as e:
                         RNS.log("The interface \""+name+"\" could not be created. Check your configuration file for errors!", RNS.LOG_ERROR)
@@ -375,25 +484,101 @@ class Reticulum:
                     RNS.panic()
                 
 
-    def createDefaultConfig(self):
+    def __create_default_config(self):
         self.config = ConfigObj(__default_rns_config__)
         self.config.filename = Reticulum.configpath
         
         if not os.path.isdir(Reticulum.configdir):
             os.makedirs(Reticulum.configdir)
         self.config.write()
-        self.applyConfig()
+        self.__apply_config()
+
+    def rpc_loop(self):
+        while True:
+            try:
+                rpc_connection = self.rpc_listener.accept()
+                call = rpc_connection.recv()
+
+                if "get" in call:
+                    path = call["get"]
+
+                    if path == "interface_stats":
+                        rpc_connection.send(self.get_interface_stats())
+
+                    if path == "next_hop_if_name":
+                        rpc_connection.send(self.get_next_hop_if_name(call["destination_hash"]))
+
+                    if path == "next_hop":
+                        rpc_connection.send(self.get_next_hop(call["destination_hash"]))
+
+                rpc_connection.close()
+            except Exception as e:
+                RNS.log("An error ocurred while handling RPC call from local client: "+str(e), RNS.LOG_ERROR)
+
+    def get_interface_stats(self):
+        if self.is_connected_to_shared_instance:
+            rpc_connection = multiprocessing.connection.Client(self.rpc_addr, authkey=self.rpc_key)
+            rpc_connection.send({"get": "interface_stats"})
+            response = rpc_connection.recv()
+            return response
+        else:
+            stats = []
+            for interface in RNS.Transport.interfaces:
+                ifstats = {}
+                
+                if hasattr(interface, "clients"):
+                    ifstats["clients"] = interface.clients
+                else:
+                    ifstats["clients"] = None
+
+                ifstats["name"] = str(interface)
+                ifstats["rxb"] = interface.rxb
+                ifstats["txb"] = interface.txb
+                ifstats["status"] = interface.online
+                stats.append(ifstats)
+
+            return stats
+
+    def get_next_hop_if_name(self, destination):
+        if self.is_connected_to_shared_instance:
+            rpc_connection = multiprocessing.connection.Client(self.rpc_addr, authkey=self.rpc_key)
+            rpc_connection.send({"get": "next_hop_if_name", "destination_hash": destination})
+            response = rpc_connection.recv()
+            return response
+        else:
+            return str(RNS.Transport.next_hop_interface(destination))
+
+    def get_next_hop(self, destination):
+        if self.is_connected_to_shared_instance:
+            rpc_connection = multiprocessing.connection.Client(self.rpc_addr, authkey=self.rpc_key)
+            rpc_connection.send({"get": "next_hop", "destination_hash": destination})
+            response = rpc_connection.recv()
+            return response
+        else:
+            return RNS.Transport.next_hop(destination)
 
-    @staticmethod
-    def should_allow_unencrypted():
-        return Reticulum.__allow_unencrypted
 
     @staticmethod
     def should_use_implicit_proof():
+        """
+        Returns whether proofs sent are explicit or implicit.
+
+        :returns: True if the current running configuration specifies to use implicit proofs. False if not.
+        """
         return Reticulum.__use_implicit_proof
 
     @staticmethod
     def transport_enabled():
+        """
+        Returns whether Transport is enabled for the running
+        instance.
+
+        When Transport is enabled, Reticulum will
+        route traffic for other peers, respond to path requests
+        and pass announces over the network.
+
+        :returns: True if Transport is enabled, False if not.
+        """
         return Reticulum.__transport_enabled
 
 # Default configuration file:
@@ -403,20 +588,12 @@ __default_rns_config__ = '''# This is the default Reticulum config file.
 
 [reticulum]
 
-# Don't allow unencrypted links by default.
-# If you REALLY need to allow unencrypted links, for example
-# for debug or regulatory purposes, this can be set to true.
-# This directive is optional and can be removed for brevity.
-
-allow_unencrypted = False
-
-
 # If you enable Transport, your system will route traffic
 # for other peers, pass announces and serve path requests.
-# Unless you really know what you're doing, this should be
-# done only for systems that are suited to act as transport
-# nodes, ie. if they are stationary and always-on. This
-# directive is optional and can be removed for brevity.
+# This should be done for systems that are suited to act
+# as transport nodes, ie. if they are stationary and
+# always-on. This directive is optional and can be removed
+# for brevity.
 
 enable_transport = False
 
@@ -434,11 +611,21 @@ share_instance = Yes
 
 
 # If you want to run multiple *different* shared instances
-# on the same system, you will need to specify a different
-# shared instance port for each. The default is given below,
-# and again, this option is optional and can be left out.
+# on the same system, you will need to specify different
+# shared instance ports for each. The defaults are given
+# below, and again, these options can be left out if you
+# don't need them.
 
 shared_instance_port = 37428
+instance_control_port = 37429
+
+# You can configure Reticulum to panic and forcibly close
+# if an unrecoverable interface error occurs, such as the
+# hardware device for an interface disappearing. This is
+# an optional directive, and can be left out for brevity.
+# This behaviour is disabled by default.
+
+panic_on_interface_error = No
 
 
 [logging]
@@ -464,11 +651,11 @@ loglevel = 4
 [interfaces]
 
   # This interface enables communication with other
-  # Reticulum nodes on your local ethernet networks.
-  # It's enabled by default, and provides basic
-  # connectivity to other peers in your local ethernet
-  # broadcast domain. You can modify it to suit your
-  # needs or turn it off completely.
+  # local Reticulum nodes over UDP. You can modify it
+  # to suit your needs or turn it off completely.
+  # As a minimum, you should probably specify the
+  # network device you want to communicate on, such
+  # as eth0 or wlan0.
   
   [[Default UDP Interface]]
     type = UDPInterface
@@ -479,6 +666,38 @@ loglevel = 4
     forward_ip = 255.255.255.255
     forward_port = 4242
 
+    # The above configuration will allow communication
+    # within the local broadcast domains of all local
+    # IP interfaces. This is enabled by default as an
+    # easy way to get started, but you might want to
+    # consider altering it to something more specific.
+
+    # Instead of specifying listen_ip, listen_port,
+    # forward_ip and forward_port, you can also bind
+    # to a specific network device like below.
+
+    # device = eth0
+    # port = 4242
+
+    # Assuming the eth0 device has the address
+    # 10.55.0.72/24, the above configuration would
+    # be equivalent to the following manual setup.
+    # Note that we are both listening and forwarding to
+    # the broadcast address of the network segments.
+
+    # listen_ip = 10.55.0.255
+    # listen_port = 4242
+    # forward_ip = 10.55.0.255
+    # forward_port = 4242
+
+    # You can of course also communicate only with
+    # a single IP address
+
+    # listen_ip = 10.55.0.15
+    # listen_port = 4242
+    # forward_ip = 10.55.0.16
+    # forward_port = 4242
+
 
   # This example demonstrates a TCP server interface.
   # It will listen for incoming connections on the
@@ -488,9 +707,23 @@ loglevel = 4
     type = TCPServerInterface
     interface_enabled = False
     outgoing = True
+
+    # This configuration will listen on all IP
+    # interfaces on port 4242
+    
     listen_ip = 0.0.0.0
     listen_port = 4242
 
+    # Alternatively you can bind to a specific IP
+    
+    # listen_ip = 10.0.0.88
+    # listen_port = 4242
+
+    # Or a specific network device
+    
+    # device = eth0
+    # port = 4242
+
 
   # To connect to a TCP server interface, you would
   # naturally use the TCP client interface. Here's
